@@ -10,7 +10,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)options.c	10.46 (Berkeley) 6/26/96";
+static const char sccsid[] = "@(#)options.c	10.49 (Berkeley) 9/15/96";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -135,6 +135,8 @@ OPTLIST const optlist[] = {
 	{"optimize",	NULL,		OPT_1BOOL,	0},
 /* O_PARAGRAPHS	    4BSD */
 	{"paragraphs",	f_paragraph,	OPT_STR,	0},
+/* O_PATH	  4.4BSD */
+	{"path",	NULL,		OPT_STR,	0},
 /* O_PRINT	  4.4BSD */
 	{"print",	f_print,	OPT_STR,	0},
 /* O_PROMPT	    4BSD */
@@ -215,7 +217,7 @@ OPTLIST const optlist[] = {
 /* O_WINDOW	    4BSD */
 	{"window",	f_window,	OPT_NUM,	0},
 /* O_WINDOWNAME	    4BSD */
-	{"windowname",	f_windowname,	OPT_0BOOL,	0},
+	{"windowname",	NULL,		OPT_0BOOL,	0},
 /* O_WRAPLEN	  4.4BSD */
 	{"wraplen",	NULL,		OPT_NUM,	0},
 /* O_WRAPMARGIN	    4BSD */
@@ -346,6 +348,8 @@ opts_init(sp, oargs)
 	OI(O_MSGCAT, b1);
 	OI(O_REPORT, "report=5");
 	OI(O_PARAGRAPHS, "paragraphs=IPLPPPQPP LIpplpipbp");
+	(void)snprintf(b1, sizeof(b1), "path=%s", "");
+	OI(O_PATH, b1);
 	(void)snprintf(b1, sizeof(b1), "recdir=%s", _PATH_PRESERVE);
 	OI(O_RECDIR, b1);
 	OI(O_SECTIONS, "sections=NHSHH HUnhsh");
@@ -1068,7 +1072,7 @@ opts_copy(orig, sp)
 	int cnt, rval;
 
 	/* Copy most everything without change. */
-	memmove(sp->opts, orig->opts, sizeof(orig->opts));
+	memcpy(sp->opts, orig->opts, sizeof(orig->opts));
 
 	/* Copy the string edit options. */
 	for (cnt = rval = 0; cnt < O_OPTIONCOUNT; ++cnt) {
