@@ -12,7 +12,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)tcl.c	8.11 (Berkeley) 4/29/96";
+static const char sccsid[] = "@(#)tcl.c	8.14 (Berkeley) 8/10/96";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -183,7 +183,7 @@ tcl_gline(clientData, interp, argc, argv)
 	if ((line = malloc(len + 1)) == NULL)
 		exit(1);				/* XXX */
 	memmove(line, p, len);
-	line[len] = NULL;
+	line[len] = '\0';
 	Tcl_SetResult(interp, line, TCL_DYNAMIC);
 	return (TCL_OK);
 }
@@ -501,9 +501,9 @@ tcl_iscreen(clientData, interp, argc, argv)
 	int argc;
 	char **argv;
 {
-	SCR *sp;
+	SCR *sp, *nsp;
 	void (*scr_msg) __P((SCR *, mtype_t, char *, size_t));
-	int id, rval;
+	int rval;
 
 	if (argc != 2 && argc != 3) {
 		Tcl_SetResult(interp,
@@ -514,13 +514,13 @@ tcl_iscreen(clientData, interp, argc, argv)
 	if (getscreenid(interp, &sp, argv[1], NULL))
 		return (TCL_ERROR);
 	INITMESSAGE;
-	rval = api_iscreen(sp, argv[2], &id);
+	rval = api_edit(sp, argv[2], &nsp, 1);
 	ENDMESSAGE;
 
 	if (rval)
 		return (TCL_ERROR);
 
-	(void)sprintf(interp->result, "%d", id);
+	(void)sprintf(interp->result, "%d", nsp->id);
 	return (TCL_OK);
 }
 
