@@ -10,7 +10,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)vi.c	10.52 (Berkeley) 6/30/96";
+static const char sccsid[] = "@(#)vi.c	10.55 (Berkeley) 8/17/96";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -70,7 +70,6 @@ vi(spp)
 	SCR *next, *sp;
 	VICMD cmd, *vp;
 	VI_PRIVATE *vip;
-	u_int flags;
 	int comcount, mapped, rval;
 
 	/* Get the first screen. */
@@ -88,6 +87,9 @@ vi(spp)
 	if (v_init(sp))
 		return (1);
 
+	/* Set the focus. */
+	(void)sp->gp->scr_rename(sp, sp->frp->name, 1);
+
 	for (vip = VIP(sp), rval = 0;;) {
 		/* Resolve messages. */
 		if (!MAPPED_KEYS_WAITING(sp) && vs_resolve(sp, 0))
@@ -260,10 +262,14 @@ gc_event:
 			if (next == NULL)
 				break;
 
-			/* Switch, and display a status line. */
+			/* Switch screens, change focus. */
 			sp = next;
 			vip = VIP(sp);
+			(void)sp->gp->scr_rename(sp, sp->frp->name, 1);
+
+			/* Don't trust the cursor. */
 			F_SET(vip, VIP_CUR_INVALID);
+
 			continue;
 		}
 
@@ -372,14 +378,14 @@ intr:			CLR_INTERRUPT(sp);
 
 			/*
 			 * If the current screen is still displayed, it will
-			 * want a new status line.
+			 * need a new status line.
 			 */
-			if (F_ISSET(sp, SC_STATUS) && vs_resolve(sp, 0))
-				goto ret;
+			F_SET(sp, SC_STATUS);
 
-			/* Switch screens. */
+			/* Switch screens, change focus. */
 			sp = sp->nextdisp;
 			vip = VIP(sp);
+			(void)sp->gp->scr_rename(sp, sp->frp->name, 1);
 
 			/* Don't trust the cursor. */
 			F_SET(vip, VIP_CUR_INVALID);
@@ -389,8 +395,11 @@ intr:			CLR_INTERRUPT(sp);
 				return (1);
 		}
 
-		/* If the last command switched files, we don't care. */
-		F_CLR(sp, SC_FSWITCH);
+		/* If the last command switched files, change focus. */
+		if (F_ISSET(sp, SC_FSWITCH)) {
+			F_CLR(sp, SC_FSWITCH);
+			(void)sp->gp->scr_rename(sp, sp->frp->name, 1);
+		}
 
 		/* If leaving vi, return to the main editor loop. */
 		if (F_ISSET(gp, G_SRESTART) || F_ISSET(sp, SC_EX)) {
